#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "vector.h"
#include "matrix.h"
#include "quaternion.h"

#define M_PI 3.14159265359

void rotate_vector(float rad, vec3 &vec, vec3 &axis)
{
	matrix3	m;
	float	sin_val, cos_val, minus_val;

	sin_val = (float)sin(rad);
	cos_val = (float)cos(rad);
	minus_val = 1.0f - cos_val;

	m.m[0] = cos_val + minus_val * axis.x * axis.x;
	m.m[1] = minus_val * axis.x * axis.y - sin_val * axis.z;
	m.m[2] = minus_val * axis.x * axis.z + sin_val * axis.y;

	m.m[3] = minus_val * axis.y * axis.x + sin_val * axis.z;
	m.m[4] = cos_val + minus_val * axis.y * axis.y;
	m.m[5] = minus_val * axis.y * axis.z - sin_val * axis.x;

	m.m[6] = minus_val * axis.z * axis.x - sin_val * axis.y;
	m.m[7] = minus_val * axis.z * axis.y + sin_val * axis.x;
	m.m[8] = cos_val + minus_val * axis.z * axis.z;

	vec = m * vec;
}

int main(void)
{
	quaternion p(0.0, 0.0, 0.0, 1.0);
	quaternion q(cos(-M_PI/8.0), sin(-M_PI/8.0), 0.0, 0.0);
	quaternion r(0.0, 0.0, 0.0, 0.0);
	quaternion up(0.0, 0.0, 1.0, 0.0);
	quaternion forwardq(0.0, 0.0, 0.0, 1.0);
	quaternion right(0.0, 1.0, 0.0, 0.0);
	matrix3		m;

	vec3 forward(0.0, 0.0, 1.0);
	vec3 xaxis(1.0, 0.0, 0.0);

	r = q*p;
	r = r*q.conjugate();

	printf("Rotating vector (0.0, 0.0, 1.0) about the (1.0, 0.0, 0.0) axis by 45 degrees\n");
	printf("(Forward vector rotated up 45 degrees)\n");
	printf("Result: (%f, (%f, %f, %f))\n", r.s, r.x, r.y, r.z);

	printf("Rotating vector using different method.\n");
	rotate_vector(M_PI/4.0, forward, xaxis);
	printf("Result: (%f, %f, %f)\n\n", forward.x, forward.y, forward.z);

	printf("Interpolating forward (0.0, 0.0, 1.0) to up (0.0, 1.0, 0.0) in 10 steps\n");
	for(float t = 0.0; t < 1.1; t += 0.1)
	{
		quaternion::slerp(p, up, t, r);
		printf("Result: t = %1.1f (%f, (%f, %f, %f))\n", t, r.s, r.x, r.y, r.z);
	}

	printf("\nDemonstrating result of converting quaternions to rotation matrices\n");
	printf("forward q(0.0, (0.0, 0.0, 1.0)) becomes:\n");
	m = forwardq.to_matrix();

	for(int i = 0; i < 9; i++)
	{
		printf("%1.1lf ", m.m[i]);
		if (i == 2 || i == 5 || i == 8)
			printf("\n");
	}

	printf("up q(0.0, (0.0, 1.0, 0.0)) becomes:\n");
	m = up.to_matrix();

	for(int i = 0; i < 9; i++)
	{
		printf("%1.1lf ", m.m[i]);
		if (i == 2 || i == 5 || i == 8)
			printf("\n");
	}

	printf("right q(0.0, (1.0, 0.0, 0.0)) becomes:\n");
	m = right.to_matrix();

	for(int i = 0; i < 9; i++)
	{
		printf("%1.1lf ", m.m[i]);
		if (i == 2 || i == 5 || i == 8)
			printf("\n");
	}




	return 0;
}